/**
 * @file	nsarray.h
 * @brief	#NSArray class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #NSArray class definition.
 *   #NSArray operates fixed size array.
 */

#ifndef NSARRAY_H_INCLUDED
#define NSARRAY_H_INCLUDED

#include <Cacao/nsobject.h>

/**
 * @class	NSArray
 * @brief	NSArray class operates array of objects
 * @par Description
 *   NSArray class operates array of objects.
 */
@interface NSArray : NSObject 
{
	/** Array body */
	GPtrArray *	array_body ;
}

  /**
   * @brief	Make the array which has an object.
   * @param	obj Source object.
   * @par Description
   *   Make the array which has an object.
   */
  + arrayWithObject: (id) obj ;

  /**
   * @brief	Make the array which has some objects.
   * @param	obj List of objects.
   * @par Description
   *   Make the array which has some objects.
   */
  + arrayWithObjects: (id) obj, ... ;

  /**
   * @brief	Initialize array by an argument.
   * @result	The object itself.
   * @par Description
   *   Initialize array by an argument.
   */
  - initWithObject: (id) obj ;

  /**
   * @brief	Initialize array by some arguments.
   * @result	The object itself.
   * @par Description
   *   Initialize array by some arguments.
   */
  - initWithObjects: (id) obj, ... ;

  /**
   * @brief	Initialize array by vector of objects.
   * @result	The object itself.
   * @param	objlist Vector of objects.
   * @param	count Number of items in the vector.
   * @par Description
   *   Initialize array by vector of objects.
   */
  - initWithObjects: (id *) objlist count: (uint_t) count ;

  /**
   * @brief	Generate array by GPtrArray data.
   * @param	garray Source garray data.
   * @par Description
   *    Generate array by GPtrArray data.
   *    This method is depend on GLib and not defined in Cocoa library.
   * @warning
   *    Do not call this method from the outside of this library.
   */
  - initWithGArray: (GPtrArray *) garray ;

  /**
   * @brief	Release the array and it's contents.
   * @par Description
   *   Release the array and it's contents.
   */
  - (void) dealloc ;

  /**
   * @brief	Get number of items in an array.
   * @result	Number of items.
   * @par Description
   *   Get number of items in an array.
   */
  - (uint_t) count ;

  /**
   * @brief	Get nth object in the array.
   * @param	index Index number of item to get.
   * @par Description
   *   Get nth object in the array.
   * @warning
   *   It's NOT allowed to access the out of range of the array.
   */
  - objectAtIndex: (uint_t) index ;

  /**
   * @brief	Get content data.
   * @return	Content data type.
   * @par Description
   *    Get content data. This method is <b>NOT</b> defined in
   *    Cocoa library. Do not call this from the other library.
   * @warning
   *    This method depends on GLib. <b>Do not</b> call this from
   *    the extenal library and programs.
   */
  - (GPtrArray *) gArray ;

@end

#endif /* NSARRAY_H_INCLUDED */

