/**
 * @file	nsstring.h
 * @brief	#NSString class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #NSString class definition.
 *   #NSString operates unicode string data.
 */

#ifndef NSSTRING_H_INCLUDED
#define NSSTRING_H_INCLUDED

#include <Cacao/nsobject.h>
#include <Cacao/nscharset.h>
#include <Cacao/nsrange.h>

/** 
 * Encoding type of string.
 * These <b>must have</b> compatibility with Cocoa.
 */
typedef enum {
	/** Encoding: Ascii */
	NSASCIIStringEncoding		= 1,
	/** Encoding: NextStep */
	NSNEXTSTEPStringEncoding	= 2,
	/** Encoding: Japanese EUC */
	NSJapaneseEUCStringEncoding	= 3,
	/** Encoding: UTF8 */
	NSUTF8StringEncoding		= 4,
	/** Encoding: ISO latin 1 */
	NSISOLatin1StringEncoding	= 5,
	/** Encoding: Symbol */
	NSSymbolStringEncoding		= 6,
	/** Encoding: Lossy ascii */
	NSNonLossyASCIIStringEncoding	= 7,
	/** Encoding: Shift JIS */
	NSShiftJISStringEncoding	= 8,
	/** Encoding: ISO latin 2 */
	NSISOLatin2StringEncoding	= 9,
	/** Encoding: Unicode */
	NSUnicodeStringEncoding		= 10,
	/** Encoding: Windoze CP 1251  */
	NSWindowzeCP1251StringEncoding	= 11,
	/** Encoding: Windoze CP 1252  */
	NSWindowzeCP1252StringEncoding	= 12,
	/** Encoding: Windoze CP 1253  */
	NSWindowzeCP1253StringEncoding	= 13,
	/** Encoding: Windoze CP 1254  */
	NSWindowzeCP1254StringEncoding	= 14,
	/** Encoding: Windoze CP 1250  */
	NSWindowzeCP1250StringEncoding	= 15,
	/** Encoding: ISO 2022 JP  */
	NSISO2022JPStringEncoding	= 21,
	/** Encoding: MacOS Roman */
	NSMacOSRomanStringEncoding	= 30,
	/** Encoding: Proprietary */
	NSProprietaryStringEncoding	= 65536
} NSStringEncoding ;

/** Imported class names */
@class	NSMutableArray, NSData ;

/**
 * @brief	Unicode string operation class.
 * @par Description
 *   #NSString operates unicode string.
 */
@interface NSString : NSObject 
{
	/** Main string data */
	GString *	str_body ;
}

  /**
   * @brief	Initialize string object.
   * @return	Generated object.
   * @par Description
   *   Initialize unicode string object.
   *   The empty string is set.
   */
  - init ;

  /**
   * @brief	Initialize string object with given C-style string.
   * @return	Generated object.
   * @param	str	Source C string.
   * @par Description
   *   Initialize string object with given C-style string.
   */
  - initWithCString: (const char *) str ;

  /**
   * @brief	Initialize string object with given C-style string and length
   * @return	Generated string.
   * @param 	str Source C string.
   * @param	length Length of source string.
   * @par Description
   *   Initialize string object with given C-style string.
   */
  - initWithCString: (const char *) str length: (size_t) length ;

  /**
   * @brief	Initialize string object with data and encoding info.
   * @return	Generated string.
   * @param 	data Source encoding.
   * @param	typ Encoding type.
   * @par Description
   *   Initialize string object with data and encoding info.
   */
  - initWithData: (NSData *) data encoding: (NSStringEncoding) typ ;

  /**
   * @brief	Initiaize string by GString of glib.
   * @return	New string.
   * @param	gstr Source string.
   * @par Description
   *    Initialize string by GString of glib.
   *    The copy is <b>NOT</b> copid in this method,
   *    but released destructor of this object.
   *    Do not free the source string.
   * @warning
   *    This method does not have compatibility with Cocoa.
   *    Do not call this except internal use.
   */
  - initWithGString: (GString *) gstr ;

  /**
   * @brief	Release resource for the object.
   * @par Description
   *   Release resource for the object.
   */
  - (void) dealloc ;

  /**
   * @brief	Get length of the string.
   * @return	Length of string.
   * @par Description
   *    Get length of the string.
   */
  - (size_t) length ;

  /**
   * @brief	Get size of the string (by bytes).
   * @return	Size of string.
   * @par Description
   *   Get size of the string (by bytes).
   */
  - (size_t) cStringLength ;

  /**
   * @brief	Get indexed character.
   * @return	Unicode at the index.
   * @param	index Index.
   * @par Description
   *   Get indexed character.
   */
  - (unichar_t) characterAtIndex: (uint_t) index ;

  /**
   * @brief	Get range of substring in the string.
   * @return	Range of substring. 
   *		If it is not found, this value will be {#NSNotFound, 0}.
   * @param	substr Substring.
   * @par Description
   */
  - (NSRange) rangeOfString: (NSString *) substr ;

  /**
   * @brief	Get range of substring in the string.
   * @return	Range of substring. 
   *		If it is not found, this value will be {#NSNotFound, 0}.
   * @param	substr Substring.
   * @param	mask Bit mask generated by #NSStringSearchOption values.
   * @par Description
   */
  - (NSRange) rangeOfString: (NSString *) substr options: (uint_t) mask ;

  /**
   * @brief	Get range of substring in the string.
   * @return	Range of substring. 
   *		If it is not found, this value will be {#NSNotFound, 0}.
   * @param	substr Substring.
   * @param	mask Bit mask generated by #NSStringSearchOption values.
   * @par Description
   */
  - (NSRange) rangeOfString: (NSString *) substr options: (uint_t) mask
      range: (NSRange) range ;

  /**
   * @brief	Compare 2 string object.
   * @return	Result of comparison
   * @param	src Source string.
   * @par Description
   *   This method compares the string and source string.
   *   If this object is smaller than source,
   *   returns #NSOrderedAscending .
   * @warning
   *   Do not pass nil as <code>src</code>.
   */
  - (NSComparisonResult) compare: (NSString *) src ;

  /**
   * @brief	Add the string to the end of string.
   * @return	New string contains result of addition.
   * @param	src Source string to append.
   * @par Description
   *   Add the string to the end of string.
   */
  - (NSString *) stringByAppendingString: (NSString *) src ;

  /**
   * @brief	Get substring from source string.
   * @return	Substring of original
   * @param	range Range of target substring.
   * @par Description
   *   Get substring from source string.
   *   Invalid range causes system error.
   */
  - (NSString *) substringWithRange: (NSRange) range ;

  /**
   * @brief	Pad the string until given length
   * @return	Generated string.
   * @param	newlength The length of new string.
   * @param	padstr Addtional string to be added to the end of
   *		current string.
   * @param	padindex The start point to add the padstr to the
   *		current string. If the added length is not enough to the
   *		<code>newlength</code>, the padstr is added from beggining.
   */
  - (NSString *) stringByPaddingToLength: (size_t) newlength
      withString: (NSString *) padstr startingAtIndex: (uint_t) padindex ;

  /**
   * @brief	Remove head/tail characters which belongs to given set.
   *		And generate new string.
   * @param	set Character set to choose trim target.
   * @par Description
   *    Remove head/tail characters which belongs to given character set.
   */
  - (NSString *) stringByTrimmingCharactersInSet: (NSCharacterSet *) set ;

  /**
   * @brief	Separate string by given separator.
   * @return	Array of separated strings.
   * @param	separator Separator string. 
   *		DO NOT contain blanks at head and tail of the separator.
   * @par Description
   *   Separate string by given separator.
   *   Spaces for each separated string are ignored.
   */
  - (NSMutableArray *) componentsSeparatedByString: (NSString *) separator ;

  /**
   * @brief	Get pointer of string.
   * @return	Pointer of string. This NEVER be null.
   * @par Description
   *   Get pointer of string.
   */
  - (const char *) cString ;

  /**
   * @brief	Get GString data.
   * @return	GString object.
   * @par Description
   *    Get GString data. GString is data structure defined by glib.
   *    This method is <b>custom method</b> for Cacao library.
   *    Do not call this from outside.
   * @warning
   *    This method does not have compatibility with Cocoa.
   *    Do not call this except internal use.
   */
  - (GString *) gString ;

@end

#endif /* NSSTRING_H_INCLUDED */

